<?php

namespace Litecms\Person\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Person\Http\Requests\PersonRequest;
use Litecms\Person\Interfaces\PersonRepositoryInterface;
use Litecms\Person\Models\Person;

/**
 * Resource controller class for person.
 */
class PersonResourceController extends BaseController
{

    /**
     * Initialize person resource controller.
     *
     * @param type PersonRepositoryInterface $person
     *
     * @return null
     */
    public function __construct(PersonRepositoryInterface $person)
    {
        parent::__construct();
        $this->repository = $person;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Person\Repositories\Criteria\PersonResourceCriteria::class);
    }

    /**
     * Display a list of person.
     *
     * @return Response
     */
    public function index(PersonRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Person\Repositories\Presenter\PersonPresenter::class)
                ->$function();
        }

        $people = $this->repository->paginate();

        return $this->response->title(trans('person::person.names'))
            ->view('person::person.index', true)
            ->data(compact('people'))
            ->output();
    }

    /**
     * Display person.
     *
     * @param Request $request
     * @param Model   $person
     *
     * @return Response
     */
    public function show(PersonRequest $request, Person $person)
    {

        if ($person->exists) {
            $view = 'person::person.show';
        } else {
            $view = 'person::person.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('person::person.name'))
            ->data(compact('person'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new person.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PersonRequest $request)
    {

        $person = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('person::person.name')) 
            ->view('person::person.create', true) 
            ->data(compact('person'))
            ->output();
    }

    /**
     * Create new person.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PersonRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $person                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('person::person.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('person/person/' . $person->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/person/person'))
                ->redirect();
        }

    }

    /**
     * Show person for editing.
     *
     * @param Request $request
     * @param Model   $person
     *
     * @return Response
     */
    public function edit(PersonRequest $request, Person $person)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('person::person.name'))
            ->view('person::person.edit', true)
            ->data(compact('person'))
            ->output();
    }

    /**
     * Update the person.
     *
     * @param Request $request
     * @param Model   $person
     *
     * @return Response
     */
    public function update(PersonRequest $request, Person $person)
    {
        try {
            $attributes = $request->all();

            $person->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('person::person.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('person/person/' . $person->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('person/person/' . $person->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the person.
     *
     * @param Model   $person
     *
     * @return Response
     */
    public function destroy(PersonRequest $request, Person $person)
    {
        try {

            $person->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('person::person.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('person/person/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('person/person/' . $person->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple person.
     *
     * @param Model   $person
     *
     * @return Response
     */
    public function delete(PersonRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('person::person.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('person/person'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/person/person'))
                ->redirect();
        }

    }

    /**
     * Restore deleted people.
     *
     * @param Model   $person
     *
     * @return Response
     */
    public function restore(PersonRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('person::person.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/person/person'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/person/person/'))
                ->redirect();
        }

    }

}
